function [xylog,linexlog,lineylog,Fits,Stdev,stdlog] = FitData(grp,fit,EQ)
%FitData(grp,fit) Generates fits from plot data
%   Kevin Mackie, 2002/01/26
%   University of California Berkeley
%   Department of Civil and Environmental Engineering
%   mackie@ce.berkeley.edu
%
%   INPUT: grp (Ax2) matrices of data in linear space
%          A corresponds to total number of input files considered
%          1st column is x (DM), 2nd column y (IM)
%          fit 1 linear 2 bilinear
%          EQ number of bins
%
%   OUTPUT: xylog - cell with *.v(:,1) as DM and *.v(:,2) as IM
%           linexlog - cell with x fit data
%           linexlog - cell with y fit data
%           Fits - coefficients of linear fit (slope, intercept)
%           Stdev - dispersion of fit
%           stdfit - mean+1sig, mean-1sig lines of fit
%
% Interpolation functions are fitting the DM given the IM (ie, y given x)
% but don't be fooled by the "reversed" axis plotting of these measures
% Output linear fits are of form log(DM)=A+B log(IM)
%
% Plots the IM DM relationship showing sensitivity of the relationship
% to design parameters.  Parameters can be varied up to 16 values.

[m,n]=size(grp);
% this makes it possible to plot arbitrary number of parameter
% variations, not just 4

for k=1:n,
   xy{k}.v=abs(grp{k}.v);
   xy{k}.v=log(xy{k}.v);
end

% sort the data to be increasing along IM axis
for k=1:n,   
   % use subset to fit neglecting first and last value
   xyp{k}.v=sortrows(xy{k}.v,[2 1]);
   xy{k}.v=xyp{k}.v(2:length(xyp{k}.v)-1,:);
   st{k}.v=[mean(xy{k}.v(1:EQ,2));mean(xy{k}.v(1:EQ,1))];
   xylog{k}.v=exp(xyp{k}.v);

   if fit == 2,
      % bilinear fit using Dimitrios' routines
      [b{k}.v,sig{k},liney{k},linex{k}] = ...
   	    	bilinear_fit(xy{k}.v(:,2),xy{k}.v(:,1),st{k}.v(1),st{k}.v(2),0);
			
	  % P fits are in form B1 (slope), A1 (intercept), B2 (slope), A2 (intercept)
      P{k}.v=[fliplr(b{k}.v(2:3)) fliplr(b{k}.v(4:5))];

	  % add back first and last data points	  
	  liney{k}=xyp{k}.v(:,2);
	  lineylog{k}=exp(liney{k});
	  
	  pfvals=cat(1,P{k}.v{:});
	  
	  fpt=pfvals(1)*xyp{k}.v(1,2)+pfvals(2);
	  lpt=pfvals(3)*xyp{k}.v(length(xyp{k}.v),2)+pfvals(4);
	  linex{k}=[fpt;linex{k};lpt];
      linexlog{k}=exp(linex{k});
	  
   elseif fit == 1,
      % polynomial least squares fit using Matlab routines
      N=1;
	  % P fits are in form B (slope), A (intercept)
      P{k}.v=polyfit(xy{k}.v(:,2),xy{k}.v(:,1),N);
	  
      liney{k}=xy{k}.v(:,2);
      linex{k}=zeros(length(liney{k}),1);

      for i=0:N,
         linex{k}=linex{k}+P{k}.v(i+1)*liney{k}.^(N-i);
      end;
	  
	  % report statistical quantities
	  Nl=length(linex{k});
	  sig{k}=sqrt( sum( ( linex{k} - xy{k}.v(:,1) ).^2 ) ./ (Nl-2) );

	  % add back first and last data points	  
	  liney{k}=xyp{k}.v(:,2);
	  lineylog{k}=exp(liney{k});
	  
	  fpt=P{k}.v(1)*xyp{k}.v(1,2)+P{k}.v(2);
	  lpt=P{k}.v(1)*xyp{k}.v(length(xyp{k}.v),2)+P{k}.v(2);
	  linex{k}=[fpt;linex{k};lpt];
      linexlog{k}=exp(linex{k});
	  
   else
      disp('Unknown fit type in ploting routine');
   end;
end;

% output fit quantities to command line, look into adding these
% to plots directly
for k=1:n,
   if iscell(P{k}.v),
      tfit=cat(1,P{k}.v{:})';
   else
      tfit=P{k}.v;
   end;
   Fits(k,1:length(P{k}.v))=tfit;
   Stdev(k)=sig{k};
   
   % calculate mean+1sigma, mean-1sigma lines if desired
   Nl=length(linex{k});
   mn=mean(linex{k});
   stdfit{k} = sqrt( sig{k}*(1+1/Nl+((linex{k}-mn).^2)./sum((linex{k}-mn).^2)) );
   stdlog{k}.p = exp(stdfit{k} + liney{k});
   stdlog{k}.m = exp(liney{k} - stdfit{k});
end
